# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import logging
from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimForeign,
    HprimString,
    HprimList,
    HprimMultiple,
    HprimDate,
    EncodeWorklistError
)


logger = logging.getLogger('driverng.elo')


class EloSegmentH(HeaderSegment):
    sender_id = HprimString(5)
    processing_id = HprimString(12, default='P')
    date = HprimDate(14)

    patients = HprimForeign('P', 'header')
    queries = HprimForeign('Q', 'header')
    end = HprimForeign('L', 'header', one2one=True, required=True)


class EloSegmentQ(Segment):
    sample_id = HprimString(3)
    universal_test_id = HprimString(5, default='ALL')
    status_code = HprimString(13, default='O')


class EloSegmentP(Segment):
    patient_id = HprimString(3)
    last_name = HprimString((6, 1))
    first_name = HprimString((6, 2))
    date_naissance = HprimDate(8, fmt_date='%Y%m%d')
    # M/F, empty if unknown
    sex = HprimString(9)

    orders = HprimForeign('O', 'patient')


class EloSegmentO(Segment):
    sample_id = HprimString((3, 1))
    reference_flacon = HprimString(4)
    universal_method_id = HprimList(5, basefield=HprimMultiple(
        0,
        code=HprimString(4),
        dilution=HprimString(5)
    ))
    dilution = HprimString((5, 5))
    date = HprimDate(7)
    specimen_descriptor = HprimString(16)
    sender_id = HprimString(17)
    # unused in mca
    # archive_position = HprimString(21)
    analyseur = HprimString(25)
    report_type = HprimString(26)

    results = HprimForeign('R', 'order')


class EloSegmentR(Segment):
    PARTIAL_RESULT_STATUS = 'S'
    OVERALL_RESULT_STATUS = 'F'

    universal_method_id = HprimString((3, 4))
    dilution = HprimString((3, 5))
    valeur = HprimString(4)
    unit = HprimString(5)
    # unused in mca
    # reference_range = HprimString(6)
    status = HprimString(9)


class EloSegmentC(Segment):
    commentaire = HprimString(3)


class EloSegmentL(Segment):
    termination_code = HprimString(3, default='N')


class Driver(AstmMessageV2):
    SPEC = {
        'code': 'elo',
        'label': 'EuroLabOffice'
    }

    SEGMENT_CLASS_MAP = {
        'H': EloSegmentH,
        'Q': EloSegmentQ,
        'P': EloSegmentP,
        'O': EloSegmentO,
        'R': EloSegmentR,
        'C': EloSegmentC,
        'L': EloSegmentL,
    }
    DEFAULT_SEPARATORS = '|\^&'

    OPTIONS = [
        AstmMessageV2.Option("elo_dilution_default", label="Facteur de dilution par défaut"),
        AstmMessageV2.BooleanOption(
            "elo_ignore_partial_results",
            label="Ignorer les résultats partiels",
            default=False)
    ]

    def __init__(self, *args, **kwargs):
        self.elo_dilution_default = kwargs.pop('elo_dilution_default', None)
        self.elo_ignore_partial_results = kwargs.pop('elo_ignore_partial_results', None)
        super(Driver, self).__init__(*args, **kwargs)

    def create(self, **kwargs):
        message = EloSegmentH(type_segment='H', **kwargs)
        message.end = EloSegmentL(type_segment='L')
        return message

    def _check_order_worklist(self, worklist):
        for sample in worklist.samples:
            if not sample.sample_id:
                raise EncodeWorklistError('Tentative d\'envoie d\'un message ne contenant pas d\'Id d\'echantillon')
            for order in sample.orders:
                if not order.code:
                    raise EncodeWorklistError(
                        'Tentative d\'envoie d\'un message contenant un test sans code demande')
        return True

    def encode_worklist(self, worklist, context):
        try:
            if worklist.context == 'order':
                report_type = 'O'
            else:
                raise EncodeWorklistError('Context invalide. Seul "order" est accepté.')
            self._check_order_worklist(worklist)
            msg = self.create(
                date=context['now']
            )
            if not msg.sender_id:
                msg.sender_id = worklist.sender_id
            astm_worklist = worklist.get_astm_worklist(
                unique_together=['action'],
                action={'del': worklist.IGNORE, worklist.ELSE: ''}
            )
            if not astm_worklist:
                msg.patients.add()
                msg.end.termination_code = 'I'
            else:
                for patient in astm_worklist:
                    if not msg.sender_id:
                        msg.sender_id = worklist.sender_id
                    sex = patient.sex if patient.sex in ['F', 'M'] else None
                    segment_p = msg.patients.add(
                        patient_id=patient.patient_id,
                        last_name=patient.last_name,
                        first_name=patient.first_name,
                        date_naissance=patient.birthdate,
                        sex=sex)
                    for sample, action in patient.samples:
                        segment_o = segment_p.orders.add(
                            sample_id=sample.sample_id,
                            date=sample.request_date,
                            specimen_descriptor=sample.specimen,
                            report_type=report_type)
                        for order in sample.orders:
                            if self.elo_dilution_default is not None and order.dilution is None:
                                dilution = self.elo_dilution_default
                            else:
                                dilution = order.dilution
                            umi = segment_o.universal_method_id.add()
                            umi.code = order.code
                            umi.dilution = dilution
        except EncodeWorklistError as e:
            logger.error(e.message)
            msg = self.create(
                date=context['now'])
            msg.end.termination_code = 'E'
        return msg

    def decode_message(self, message):
        if message.queries:
            worklist = Worklist(context='query')
            for query in message.queries:
                sample = worklist.get_or_create(
                    type='patient',
                    sample_id=query.sample_id)
        else:
            worklist = Worklist(
                context='result',
                sender_id=message.sender_id)
            for patient in message.patients:
                for order in patient.orders:
                    if order.report_type == 'Q':
                        sample = worklist.get_or_create(
                            sample_id=order.reference_flacon,
                            type='control',
                            control_lot=order.reference_flacon)
                        for result in order.results:
                            sample.results.add(
                                code=result.universal_method_id,
                                value=result.valeur,
                                instrument=order.analyseur)
                    else:
                        sample = worklist.get_or_create(
                            type='patient',
                            sample_id=order.sample_id,
                            last_name=patient.last_name,
                            first_name=patient.first_name,
                            birthdate=patient.date_naissance,
                            sex=patient.sex,
                            specimen=order.specimen_descriptor)
                        for result in order.results:
                            if self.elo_ignore_partial_results and result.status == result.PARTIAL_RESULT_STATUS:
                                continue
                            current_result = sample.results.add(
                                code=result.universal_method_id,
                                dilution=result.dilution,
                                value=result.valeur,
                                unit=result.unit,
                                instrument=order.analyseur,
                                status=result.status or order.report_type)
                            for comment in result.commentaires:
                                current_result.comments.append(comment.commentaire)

        return worklist
